package scg.ac.ni.pmf.newgraph.plugin.dpd;

import java.util.Collection;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.Vector;

import scg.ac.ni.pmf.newgraph.dpd.DPD;
import scg.ac.ni.pmf.newgraph.graphdata.Graph;
import scg.ac.ni.pmf.newgraph.graphdata.GraphData;
import scg.ac.ni.pmf.newgraph.graphdata.Vertex;
import scg.ac.ni.pmf.newgraph.graphdata.VertexData;
import scg.ac.ni.pmf.newgraph.graphdata.WeightVertexInt;
import scg.ac.ni.pmf.newgraph.param.Parametrizable;

/**
 * <p>Separates all the vertices into two families of sets.
 * Alpha family are the vertices that were generated when
 * the weights on vertices are transformed into coctail
 * party groups. Each alpha group is a coctail party
 * graph. Alpha groups could be empty, for instance when the 
 * vertex had label zero. Beta family are vertices that represent
 * edges in the root graph. Each beta family is a complete graph.
 * Each beta vertex belongs to one or two beta groups.
 * All beta neighbors of a beta vertex belong to the beta groups
 * that the beta vertex belongs to. Each beta group is assigned exactly one
 * alpha group (possibly empty), and vice versa. All vertices between the associated
 * beta and alpha groups are connected.</p>
 * 
 * <p>The algorithm tries to find two connected vertices with the
 * biggest number of mutual neighbors. They will be used to find the
 * first beta, and the associated alpha, groups.
 * There might be one mutual neighbor which
 * is not connected to any of other neighbors. Of the other neighbors,
 * </p>
 * 
 * @author <a href="mailto: brankovv@beotel.yu">Vladimir Brankov</a>
 */
public class GeneralizedLineGraphRoot2 implements DPD, Parametrizable {

	public void create(Graph dest) {
		Collection components = separateComponents(source);
		
		for(Iterator i = components.iterator(); i.hasNext(); ) {
			Set component = (Set) i.next();
			if(isCP(component))
				buildCoctailParty(dest, component);
			else if(isSimple(component))
				buildSimple(dest, component);
			else {
				CI ci = new CI();
				if(!ci.buildComplex(source, dest, component)) {
					dest.selectAll();
					dest.deleteSelection();
					break;
				}
			}
		}
	}
	
	private class CI {
		
		public Set as = new HashSet(), bs = new HashSet(),
			pp = new HashSet(), tp = new HashSet();
		public Map atob = new HashMap(), btoa = new HashMap(),
			vtob1 = new HashMap(), vtob2 = new HashMap(),
			vtoa = new HashMap();
		
		/**
		 * If the source graph has at least two edges, builds the root graph.
		 * 
		 * @param source
		 * @param dest
		 * @param component
		 * @return
		 */
		private boolean buildComplex(GraphData source, Graph dest, Collection component) {
			Set a = new HashSet(), b = new HashSet();
			// find an initial a b pair
			if(!findInitialAB(component, a, b))
				return false;
//			System.out.println("a = " + debugSetVertex(a) + ", b = " + debugSetVertex(b));
			
			addAB(a, b);
			tp.addAll(b);
			
			while(!tp.isEmpty()) {
				VertexData u = (VertexData) tp.iterator().next();
				tp.remove(u);
				if(pp.contains(u))
					continue;

				a = new HashSet();
				b = new HashSet();
				if(!separateAB(u, a, b))
					return false;
				
				pp.add(u);
				tp.addAll(b);
			}
			
			// create vertices
			Map btov = new HashMap();
			for(Iterator i = bs.iterator(); i.hasNext(); ) {
				b = (Set) i.next();
				a = (Set) btoa.get(b);
				Pt pt = null;
				if(a.size() > 0)
					pt = findCenter(a);
				else
					pt = findCenter(b);
				Vertex v = dest.createVertex(pt.x, pt.y);
				btov.put(b, v);
				dest.addWeight(v.getData(), weight, a.size() / 2);
			}
			
			// connect vertices
			for(Iterator i = bs.iterator(); i.hasNext(); ) {
				Set b1 = (Set) i.next();
				Vertex u = (Vertex) btov.get(b1);
				for(Iterator j = bs.iterator(); j.hasNext(); ) {
					Set b2 = (Set) j.next();
					Vertex v = (Vertex) btov.get(b2);
					if(haveCommonVertices(b1, b2))
						dest.createEdge(u, v);
				}
			}
			
			return true;
		}
		
		private boolean separateAB(VertexData u, Set a, Set b) {
//			System.out.println("separateAB(u = " + u.getID() + ")");
			Set s = findNeighbors(u);
			if(!vtob1.containsKey(u))
				return false;
			Set b1 = (Set) vtob1.get(u);
			s.removeAll(b1);
			s.removeAll((Set) btoa.get(b1));
			s.add(u);
			if(vtob2.containsKey(u)) {
				Set b2 = (Set) vtob2.get(u);
				s.removeAll((Set) btoa.get(b2));
				return s.equals(b2);
			}
			
			// of all the neighbors, find the set of those with least degree
			int d = -1;
			for(Iterator i = s.iterator(); i.hasNext(); ) {
				VertexData v = (VertexData) i.next();
				if(d == -1 || v.degree() < d)
					d = v.degree();
			}
			Set l = new HashSet();
			for(Iterator i = s.iterator(); i.hasNext(); ) {
				VertexData v = (VertexData) i.next();
				if(v.degree() == d)
					l.add(v);
			}
			if(isCP(l)) {
				a.addAll(l);
				b.addAll(s);
				b.removeAll(l);
			} else {
				b.addAll(s);
			}
			return addAB(a, b);
		}
		
		private boolean addAB(Set a, Set b) {
//			System.out.println("addAB(" + debugSetVertex(a) + ", " + debugSetVertex(b) + ")");
			if(!checkAB(a, b))
				return false;
			
			// check that every vertex that belongs to a can belong to only one set
			for(Iterator i = a.iterator(); i.hasNext(); ) {
				VertexData u = (VertexData) i.next();
				if(vtoa.containsKey(u) && !a.equals(vtoa.get(u)))
					return false;
				if(vtob1.containsKey(u) || vtob2.containsKey(u))
					return false;
				vtoa.put(u, a);
			}
			
			for(Iterator i = b.iterator(); i.hasNext(); ) {
				VertexData u = (VertexData) i.next();
				if(vtoa.containsKey(u))
					return false;
				if(vtob1.containsKey(u) && !vtob1.get(u).equals(b) &&
						vtob2.containsKey(u) && !vtob2.get(u).equals(b))
					return false;
				if(vtob1.containsKey(u) && !vtob1.get(u).equals(b))
					vtob2.put(u, b);
				else
					vtob1.put(u, b);
			}
			
			as.add(a);
			bs.add(b);
			atob.put(a, b);
			btoa.put(b, a);
			
			return true;
		}
		
		private boolean findInitialAB(Collection component, Set a, Set b) {
			for(Iterator i = component.iterator(); i.hasNext(); ) {
				VertexData u = (VertexData) i.next();
				for(Iterator j = component.iterator(); j.hasNext(); ) {
					VertexData v = (VertexData) j.next();
					if(!u.isAdjanced(v))
						continue;
					if(!mutualNeighborsOfLowerDegree(u, v) &&
							!(u.degree() > 1 && v.degree() > 1 && numberOfMutualNeighbors(u, v) == 0))
						continue;
					
					Vector nuv = findMutualNeighbors(u, v);
					if(nuv.size() == 0) {
						b.add(u);
						b.add(v);
						return true;
					} else if(nuv.size() == 1) {
						VertexData w = (VertexData) nuv.elementAt(0);
						if(numberOfMutualNeighbors(u, w) == 1 &&
								numberOfMutualNeighbors(v, w) == 1) {
							b.add(u);
							b.add(v);
							b.add(w);
							return true;
						}
					} else if(nuv.size() == 2) {
						VertexData x = (VertexData) nuv.elementAt(0);
						VertexData y = (VertexData) nuv.elementAt(1);
						if(x.degree() == 2 && y.degree() == 2) {
							b.add(u);
							b.add(v);
							a.add(x);
							a.add(y);
							return true;
						} else if(x.isAdjanced(y)) {
							b.add(u);
							b.add(v);
							b.add(x);
							b.add(y);
							return true;
						} else if(x.degree() == 2) {
							b.add(u);
							b.add(v);
							b.add(y);
							return true;
						} else if(y.degree() == 2) {
							b.add(u);
							b.add(v);
							b.add(x);
							return true;
						}
					} else if(nuv.size() >= 3) {
						
						// if there are two neighbors with the degree 2,
						// those are surely alpha group
						VertexData x = null, y = null;
						for(Iterator k = nuv.iterator(); k.hasNext(); ) {
							VertexData w = (VertexData) k.next();
							if(w.degree() == 2) {
								if(x == null)
									x = w;
								else if(y == null) {
									y = w;
									break;
								}
							}
						}
						if(x != null && y != null) {
							b.add(u);
							b.add(v);
							a.add(x);
							a.add(y);
							return true;
						}
						
						// if there is a vertex w which is not adjanced to any other
						// neighbor, that vertex is removed
						for(Iterator k = nuv.iterator(); k.hasNext(); ) {
							VertexData w = (VertexData) k.next();
							boolean pass = true;
							for(Iterator l = nuv.iterator(); l.hasNext(); ) {
								VertexData z = (VertexData) l.next();
								if(w.isAdjanced(z)) {
									pass = false;
									break;
								}
							}
							if(pass) {
								nuv.remove(w);
								break;
							}
						}
						
						// of the remaining neighbors, take those which are adjanced
						// to all the other neighbors, and they form beta group
						// others should form alpha group
						b.add(u);
						b.add(v);
						for(Iterator k = nuv.iterator(); k.hasNext(); ) {
							VertexData w = (VertexData) k.next();
							boolean pass = true;
							for(Iterator l = nuv.iterator(); l.hasNext(); ) {
								VertexData z = (VertexData) l.next();
								if(!w.isAdjanced(z) && w.getID() != z.getID()) {
									pass = false;
									break;
								}
							}
							if(pass)
								b.add(w);
						}
						nuv.removeAll(b);
						a.addAll(nuv);
						
						return checkAB(a, b);
					}
				}
			}
			
			// every two adjanced vertices have exactly two mutual neighbors
			VertexData u = (VertexData) component.iterator().next();
			VertexData v = null;
			for(Iterator i = component.iterator(); i.hasNext(); ) {
				VertexData w = (VertexData) i.next();
				if(u.isAdjanced(w)) {
					v = w;
					break;
				}
			}
//			System.out.println("u = " + u.getID() + ", v = " + v.getID());
			
			b.add(u);
			b.add(v);
			b.addAll(findOddTriangles(u, v));
			
			return checkAB(a, b);
		}
		
		/**
		 * Tells if there are mutual neighbors with a degree lower than the degree
		 * of the given vertices. Having such mutual neighbors signifies that the given
		 * vertices are surely beta vertices.
		 * 
		 * @param u
		 * @param v
		 * @return
		 */
		private boolean mutualNeighborsOfLowerDegree(VertexData u, VertexData v) {
			Collection mn = findMutualNeighbors(u, v);
			for(Iterator i = mn.iterator(); i.hasNext(); ) {
				VertexData w = (VertexData) i.next();
				if(w.degree() < u.degree() && w.degree() < v.degree())
					return true;
			}
			return false;
		}
		
		/**
		 * Checks the pair of alpha and beta sets.
		 * 
		 * @param a
		 * @param b
		 * @return
		 */
		private boolean checkAB(Set a, Set b) {
//			System.out.println("checkAB(a = " + debugSetVertex(a) + ", b = " + debugSetVertex(b) + ")");
			if(!isCP(a))
				return false;
			if(!isComplete(b))
				return false;
			for(Iterator i = a.iterator(); i.hasNext(); ) {
				VertexData u = (VertexData) i.next();
				if(u.degree() != a.size() - 2 + b.size())
					return false;
				for(Iterator j = b.iterator(); j.hasNext(); ) {
					VertexData v = (VertexData) j.next();
					if(!u.isAdjanced(v))
						return false;
				}
			}
			return true;
		}
		
		// debug methods
		
		public void debug() {
//			System.out.println("as = " + debugSetSetVertex(as));
//			System.out.println("bs = " + debugSetSetVertex(as));
//			System.out.println("pp = " + debugSetVertex(pp));
//			System.out.println("tp = " + debugSetVertex(tp));
		}
		
		public void debugStructure() {
//			System.out.println("atob = " + debugMapSetSet(atob));
//			System.out.println("btoa = " + debugMapSetSet(btoa));
//			System.out.println("vtob1 = " + debugMapVertexSet(vtob1));
//			System.out.println("vtob2 = " + debugMapVertexSet(vtob2));
//			System.out.println("vtoa = " + debugMapVertexSet(vtoa));
		}
		
	}
	
	// methods for simple situations
	
	/**
	 * Builds a root of the generalized line graph, if the
	 * source graph is a coctail party. The result is an isolated
	 * vertex.
	 * 
	 * @param dest
	 * @param component
	 */
	private void buildCoctailParty(Graph dest, Collection component) {
		Pt c = findCenter(component);
		Vertex v = dest.createVertex(c.x, c.y);
		dest.addWeight(v.getData(), weight, component.size() / 2);
	}

	/**
	 * Tests is the graph a vertex connected to up to two separate coctail party
	 * graphs. If so, the root graph is an edge.
	 * 
	 * @param source
	 * @param component
	 * @return
	 */
	private boolean isSimple(Collection component) {
		Set beta1 = new HashSet(), beta2 = new HashSet();
		if(simpleSeparation(component, beta1, beta2) == null)
			return false;
		return isCP(beta1) && isCP(beta2);
	}
	
	/**
	 * If a graph a vertex connected to up to two separate coctail party graphs,
	 * builds the root graph, which is an edge.
	 *  
	 * @param source
	 * @param dest
	 * @param component
	 */
	private void buildSimple(Graph dest, Collection component) {
		Set beta1 = new HashSet(), beta2 = new HashSet();
		VertexData center = simpleSeparation(component, beta1, beta2);
		
		Pt p1, p2;
		if(beta1.size() == 0) {
			p1 = new Pt(center.getX() - 10, center.getY());
			p2 = new Pt(center.getX() + 10, center.getY());
		} else if(beta2.size() == 0) {
			p1 = findCenter(beta1);
			p2 = new Pt(2 * center.getX() - p1.x, 2 * center.getY() - p1.y);
		} else {
			p1 = findCenter(beta1);
			p2 = findCenter(beta2);
		}
		
		Vertex v1 = dest.createVertex(p1.x, p1.y), v2 = dest.createVertex(p2.x, p2.y);
		dest.addWeight(v1.getData(), weight, beta1.size() / 2);
		dest.addWeight(v2.getData(), weight, beta2.size() / 2);
		dest.createEdge(v1, v2);
	}
	
	/**
	 * Separates a component into a central vertex, which is connected to all the other vertex,
	 * and two separated coctail party graphs. The coctail party graphs can be empty.
	 * 
	 * @param component The component.
	 * @param beta1 Fills the collection with the first coctail party subgraph.
	 * @param beta2 Fills the collection with the second coctail party subgraph.
	 * @return The central vertex.
	 */
	private VertexData simpleSeparation(Collection component, Set beta1, Set beta2) {
		int n = component.size();
		
		// find center
		VertexData center = null;
		for(Iterator i = component.iterator(); i.hasNext(); ) {
			VertexData v = (VertexData) i.next();
			if(v.degree() == n - 1) {
				center = v;
				break;
			}
		}
		if(center == null)
			return null;
		
		if(component.size() == 1)
			return center;
		
		// find neighbor of a least degree
		VertexData leastNeighbor = null;
		for(Iterator i = component.iterator(); i.hasNext(); ) {
			VertexData v = (VertexData) i.next();
			if(v.degree() < n - 1 && (leastNeighbor == null || leastNeighbor.degree() > v.degree()))
				leastNeighbor = v;
		}
		if(leastNeighbor == null)
			return null;
		
		// two sets of beta vertices
		beta1.add(leastNeighbor);
		
		// if the degree of the least neighbor is 1, find its partner
		if(leastNeighbor.degree() == 1) {
			VertexData partner = null;
			for(Iterator i = component.iterator(); i.hasNext(); ) {
				VertexData v = (VertexData) i.next();
				if(v.degree() == 1 && v.getID() != center.getID() && v.getID() != leastNeighbor.getID()) {
					partner = v;
					break;
				}
			}
			if(partner == null)
				return null;
			
			beta1.add(partner);
			
		// otherwise, add all least neighbor's neighbors, and their neighbors
		} else {
			VertexData firstNeighbor = null;
			for(Iterator i = component.iterator(); i.hasNext(); ) {
				VertexData v = (VertexData) i.next();
				if(v.getID() != center.getID() && v.getID() != leastNeighbor.getID() &&
						v.isAdjanced(leastNeighbor)) {
					beta1.add(v);
					if(firstNeighbor == null)
						firstNeighbor = v;
				}
			}
			if(firstNeighbor != null)
				for(Iterator i = component.iterator(); i.hasNext(); ) {
					VertexData v = (VertexData) i.next();
					if(v.getID() != center.getID() && v.getID() != leastNeighbor.getID() &&
							v.isAdjanced(firstNeighbor)) {
						beta1.add(v);
					}
				}
		}
		
		beta2.addAll(component);
		beta2.remove(center);
		beta2.removeAll(beta1);
		
		return center;
	}

	// helper methods with graphs

	/**
	 * Identifies is the component coctail party.
	 * 
	 * @param source The source graph.
	 * @param component
	 * @return
	 */
	private boolean isCP(Collection component) {
		for(Iterator i = component.iterator(); i.hasNext(); ) {
			VertexData u = (VertexData) i.next();
			int d = 0;
			for(Iterator j = component.iterator(); j.hasNext(); ) {
				VertexData v = (VertexData) j.next();
				if(u.isAdjanced(v))
					d++;
			}
			if(d != component.size() - 2)
				return false;
		}
		return true;
	}
	
	/**
	 * Tells if the specified set of vertices is a complete
	 * induced subgraph.
	 * 
	 * @param component
	 * @return
	 */
	private boolean isComplete(Collection component) {
		for(Iterator i = component.iterator(); i.hasNext(); ) {
			VertexData u = (VertexData) i.next();
			int d = 0;
			for(Iterator j = component.iterator(); j.hasNext(); ) {
				VertexData v = (VertexData) j.next();
				if(u.isAdjanced(v))
					d++;
			}
			if(d != component.size() - 1)
				return false;
		}
		return true;
	}
	
	/**
	 * Tells are there any vertices of those two sets connected.
	 * 
	 * @param a
	 * @param b
	 * @return
	 */
	private boolean haveCommonVertices(Set a, Set b) {
		for(Iterator i = a.iterator(); i.hasNext(); ) {
			VertexData u = (VertexData) i.next();
			if(b.contains(u))
				return true;
		}
		return false;
	}
	
	/**
	 * Returns a collection of sets of vertices, which are the components of the
	 * graph.
	 * 
	 * @param graph The graph.
	 * @return The collection of <code>Set</code> of <code>VertexData</code>.
	 */
	private Collection separateComponents(GraphData graph) {
		Collection components = new Vector();
		Set marked = new HashSet(), toProcess = new HashSet(), q = new HashSet();
		
		for(Enumeration i = graph.vertices(); i.hasMoreElements(); ) {
			VertexData u = (VertexData) i.nextElement();
			toProcess.add(u);
		}
		
		while(!toProcess.isEmpty()) {
			Set component = new HashSet();
			VertexData u = (VertexData) toProcess.iterator().next();
			q.add(u);
			component.add(u);
			while(!q.isEmpty()) {
				u = (VertexData) q.iterator().next();
				q.remove(u);
				marked.add(u);
				toProcess.remove(u);
				for(Enumeration i = u.getNeighbors(); i.hasMoreElements(); ) {
					VertexData v = (VertexData) i.nextElement();
					if(!marked.contains(v)) {
						q.add(v);
						component.add(v);
					}
				}
			}
			components.add(component);
		}
		
		return components;
	}

	
	/**
	 * Finds all the mutual neighbors of the specified vertices.
	 * 
	 * @param u The first vertex.
	 * @param v The second vertex.
	 * @return
	 */
	private Vector findMutualNeighbors(VertexData u, VertexData v) {
		Vector c = new Vector();
		for(Enumeration i = u.getNeighbors(); i.hasMoreElements(); ) {
			VertexData w = (VertexData) i.nextElement();
			if(w.isAdjanced(v))
				c.add(w);
		}
		return c;
	}
	
	/**
	 * Finds the neighbors of the given vertex.
	 * 
	 * @param u
	 * @return
	 */
	private Set findNeighbors(VertexData u) {
		Set s = new HashSet();
		for(Enumeration i = u.getNeighbors(); i.hasMoreElements(); )
			s.add(i.nextElement());
		return s;
	}
	
	/**
	 * Gives the number of mutual neighbors.
	 * 
	 * @param u
	 * @param v
	 * @return
	 */
	private int numberOfMutualNeighbors(VertexData u, VertexData v) {
		int j = 0;
		for(Enumeration i = u.getNeighbors(); i.hasMoreElements(); ) {
			VertexData w = (VertexData) i.nextElement();
			if(w.isAdjanced(v))
				j++;
		}
		return j;
	}
	
	/**
	 * Tells is the given triangle odd.
	 * 
	 * @param u
	 * @param v
	 * @param w
	 * @return
	 */
	private boolean oddTriangle(VertexData u, VertexData v, VertexData w) {
		if(!u.isAdjanced(v) || !u.isAdjanced(w) || !v.isAdjanced(w))
			return false;
		for(Enumeration i = u.getNeighbors(); i.hasMoreElements(); ) {
			VertexData x = (VertexData) i.nextElement();
			if(x.getID() == u.getID() ||
					x.getID() == v.getID() ||
					x.getID() == w.getID())
				continue;
			if((x.isAdjanced(v) && x.isAdjanced(w)) ||
					(!x.isAdjanced(v) && !x.isAdjanced(w)))
				return true;
		}
		for(Enumeration i = v.getNeighbors(); i.hasMoreElements(); ) {
			VertexData x = (VertexData) i.nextElement();
			if(x.getID() == u.getID() ||
					x.getID() == v.getID() ||
					x.getID() == w.getID())
				continue;
			if(!x.isAdjanced(u) && !x.isAdjanced(w))
				return true;
		}
		for(Enumeration i = w.getNeighbors(); i.hasMoreElements(); ) {
			VertexData x = (VertexData) i.nextElement();
			if(x.getID() == u.getID() ||
					x.getID() == v.getID() ||
					x.getID() == w.getID())
				continue;
			if(!x.isAdjanced(u) && !x.isAdjanced(v))
				return true;
		}
		return false;
	}

	/**
	 * Finds all vertices which build odd triangles with the given two
	 * vertices.
	 * 
	 * @param u
	 * @param v
	 * @return
	 */
	private Set findOddTriangles(VertexData u, VertexData v) {
		Collection c = findMutualNeighbors(u, v);
		Set o = new HashSet();
		for(Iterator i = c.iterator(); i.hasNext(); ) {
			VertexData w = (VertexData) i.next();
			if(w.getID() == u.getID() || w.getID() == v.getID())
				continue;
			if(oddTriangle(u, v, w))
				o.add(w);
		}
		return o;
	}
	
	// helper class pt (point)
	
	private Pt findCenter(Collection vertices) {
		if(vertices.size() == 0)
			return null;
		double x = 0, y = 0;
		for(Iterator i = vertices.iterator(); i.hasNext(); ) {
			VertexData u = (VertexData) i.next();
			x += u.getX();
			y += u.getY();
		}
		return new Pt(x / vertices.size(), y / vertices.size());
	}
	
	private class Pt {
		public Pt(double x, double y) {
			this.x = x;
			this.y = y;
		}
		public double x, y;
	}

	// debugging helper methods
	
	private String debugSetSetVertex(Set a) {
		StringBuffer s = new StringBuffer("{");
		boolean first = true;
		for(Iterator i = a.iterator(); i.hasNext(); ) {
			Set b = (Set) i.next();
			if(first)
				first = false;
			else
				s.append(", ");
			s.append(debugSetVertex(b));
		}
		s.append("}");
		return s.toString();
	}
	
	private String debugMapSetSet(Map m) {
		StringBuffer s = new StringBuffer("[");
		boolean first = true;
		for(Iterator i = m.entrySet().iterator(); i.hasNext(); ) {
			Map.Entry entry = (Map.Entry) i.next();
			if(first)
				first = false;
			else
				s.append(", ");
			s.append(debugSetVertex((Set) entry.getKey()) + " = " +
					debugSetVertex((Set) entry.getValue()));
		}
		s.append("]");
		return s.toString();
	}
	
	private String debugMapVertexSet(Map m) {
		StringBuffer s = new StringBuffer("[");
		boolean first = true;
		for(Iterator i = m.entrySet().iterator(); i.hasNext(); ) {
			Map.Entry entry = (Map.Entry) i.next();
			if(first)
				first = false;
			else
				s.append(", ");
			s.append(((VertexData) entry.getKey()).getID() + " = " +
					debugSetVertex((Set) entry.getValue()));
		}
		s.append("]");
		return s.toString();
	}
	
	private String debugSetVertex(Set s) {
		StringBuffer buf = new StringBuffer();
		buf.append("{");
		boolean first = true;
		for(Iterator i = s.iterator(); i.hasNext(); ) {
			VertexData u = (VertexData) i.next();
			if(first)
				first = false;
			else
				buf.append(", ");
			buf.append(Integer.toString(u.getID()));
		}
		buf.append("}");
		return buf.toString();
	}
	
	// plugin methods
	
	public String getName() {
		return "Generalized Line Graph Root 2";
	}

	public String getDescription() {
		return "Builds a source of a generalized line graph";
	}

	public String checkParameters() {
		return null;
	}
	
	private WeightVertexInt weight;

	public WeightVertexInt getWeight() {
		return weight;
	}
	
	public void setWeight(WeightVertexInt weight) {
		this.weight = weight;
	}
	
	private GraphData source;
	
	public GraphData getSource() {
		return source;
	}
	
	public void setSource(GraphData source) {
		this.source = source;
	}
	
}
