package scg.ac.ni.pmf.newgraph.plugin.dpd;

import java.util.Collection;
import java.util.Enumeration;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;
import java.util.Vector;

import scg.ac.ni.pmf.newgraph.dpd.DPD;
import scg.ac.ni.pmf.newgraph.graphdata.Graph;
import scg.ac.ni.pmf.newgraph.graphdata.GraphData;
import scg.ac.ni.pmf.newgraph.graphdata.Vertex;
import scg.ac.ni.pmf.newgraph.graphdata.VertexData;
import scg.ac.ni.pmf.newgraph.graphdata.WeightVertexInt;
import scg.ac.ni.pmf.newgraph.param.Parametrizable;

/**
 * Builds a root of a generalized line graph.
 * 
 * @author <a href="mailto: brankovv@beotel.yu">Vladimir Brankov</a>
 */
public class GeneralizedLineGraphRoot implements DPD, Parametrizable {

	public void create(Graph graph) {
		Collection components = buildComponents(source);
		for(Iterator i = components.iterator(); i.hasNext(); ) {
			Set component = (Set) i.next();
			if(isGCP(source, component))
				buildGCP(graph, source, component);
			else if(is2GCP(source, component))
				build2GCP(graph, source, component);
			else
				build(graph, source, component);
		}
	}
	
	private void build(Graph dest, GraphData src, Set component) {
		
	}
	
	private void build2GCP(Graph dest, GraphData src, Set component) {
		int n = component.size();
		VertexData center = null;
		for(Iterator i = component.iterator(); i.hasNext(); ) {
			VertexData u = (VertexData) i.next();
			if(u.degree() == n - 1)
				center = u;
		}
		if(center == null)
			return;
		
		Set cp1 = new HashSet(), beta1 = new HashSet(), cp2 = new HashSet(), beta2 = new HashSet(),
				excluded = new HashSet();
		excluded.add(center);
		if(!gcp2Separation(src, component, excluded, cp1, beta1, center))
			return;
		excluded.addAll(cp1);
		excluded.addAll(beta1);
		if(!gcp2Separation(src, component, excluded, cp2, beta2, center))
			return;
		excluded.addAll(cp2);
		excluded.addAll(beta2);
		if(excluded.size() != n)
			return;
		
		Point p1 = center(cp1), p2 = center(cp2);
		Vertex v1 = dest.createVertex(p1.x, p1.y), v2 = dest.createVertex(p2.x, p2.y);
		dest.addWeight(v1.getData(), weight, cp1.size() / 2);
		dest.addWeight(v2.getData(), weight, cp2.size() / 2);
		dest.createEdge(v1, v2);
		
		Set connected = new HashSet();
		for(Iterator i = beta1.iterator(); i.hasNext(); ) {
			VertexData u = (VertexData) i.next();
			VertexData v = null;
			for(Iterator j = beta2.iterator(); j.hasNext(); ) {
				VertexData w = (VertexData) j.next();
				if(u.isAdjanced(w)) {
					v = w;
					break;
				}
			}
			Vertex w = dest.createVertex(u.getX() * 2 - p1.x, u.getY() * 2 - p1.y);
			dest.createEdge(v1, w);
			if(v != null) {
				dest.createEdge(v2, w);
				connected.add(v);
			}
		}
		for(Iterator i = beta2.iterator(); i.hasNext(); ) {
			VertexData u = (VertexData) i.next();
			if(connected.contains(u))
				continue;
			Vertex w = dest.createVertex(u.getX() * 2 - p2.x, u.getY() * 2 - p2.y);
			dest.createEdge(v2, w);
		}
	}
	
	private void buildGCP(Graph dest, GraphData src, Set component) {
		int n = component.size();
		Collection cp = new Vector(), edges = new Vector();
		for(Iterator i = component.iterator(); i.hasNext(); ) {
			VertexData u = (VertexData) i.next();
			if(u.degree() == n - 2)
				cp.add(u);
			else if(u.degree() == n - 1)
				edges.add(u);
		}
		
		Point centerPt = center(cp);
		Vertex center = dest.createVertex(centerPt.x, centerPt.y);
		dest.addWeight(center.getData(), weight, cp.size() / 2);
		
		for(Iterator i = edges.iterator(); i.hasNext(); ) {
			VertexData u = (VertexData) i.next();
			Vertex v = dest.createVertex(u.getX() * 2 - centerPt.x,
					u.getY() * 2 - centerPt.y);
			dest.createEdge(center, v);
		}
	}
	
	private Point center(Collection vertices) {
		if(vertices.size() == 0)
			return new Point(0, 0);
		double x = 0, y = 0;
		for(Iterator i = vertices.iterator(); i.hasNext(); ) {
			VertexData u = (VertexData) i.next();
			x += u.getX();
			y += u.getY();
		}
		return new Point(x / vertices.size(), y / vertices.size());
	}
	
	private boolean is2GCP(GraphData source, Set component) {
		int n = component.size();
		VertexData center = null;
		for(Iterator i = component.iterator(); i.hasNext(); ) {
			VertexData u = (VertexData) i.next();
			if(u.degree() == n - 1)
				center = u;
		}
		if(center == null)
			return false;
		
		Set cp1 = new HashSet(), beta1 = new HashSet(), cp2 = new HashSet(), beta2 = new HashSet(),
				excluded = new HashSet();
		excluded.add(center);
		if(!gcp2Separation(source, component, excluded, cp1, beta1, center))
			return false;
		if(!isCP(cp1) || !isComplete(beta1))
			return false;
		excluded.addAll(cp1);
		excluded.addAll(beta1);
		if(!gcp2Separation(source, component, excluded, cp2, beta2, center))
			return false;
		if(!isCP(cp2) || !isComplete(beta2))
			return false;
		excluded.addAll(cp2);
		excluded.addAll(beta2);
		if(excluded.size() != n || !isBetaValid(beta1, beta2))
			return false;
		return true;
	}
	
	private boolean isBetaValid(Set beta1, Set beta2) {
		for(Iterator i = beta1.iterator(); i.hasNext(); ) {
			VertexData u = (VertexData) i.next();
			if(adjanced(u, beta2) > 1)
				return false;
		}
		for(Iterator i = beta2.iterator(); i.hasNext(); ) {
			VertexData u = (VertexData) i.next();
			if(adjanced(u, beta1) > 1)
				return false;
		}
		return true;
	}
	
	private boolean isComplete(Set complete) {
		for(Iterator i = complete.iterator(); i.hasNext(); ) {
			VertexData u = (VertexData) i.next();
			int d = 0;
			for(Iterator j = complete.iterator(); j.hasNext(); ) {
				VertexData v = (VertexData) j.next();
				if(u.isAdjanced(v))
					d++;
			}
			if(d != complete.size() - 1)
				return false;
		}
		return true;
	}
	
	private boolean isCP(Set cp) {
		for(Iterator i = cp.iterator(); i.hasNext(); ) {
			VertexData u = (VertexData) i.next();
			int d = 0;
			for(Iterator j = cp.iterator(); j.hasNext(); ) {
				VertexData v = (VertexData) j.next();
				if(u.isAdjanced(v))
					d++;
			}
			if(d != cp.size() - 2)
				return false;
		}
		return true;
	}
	
	private boolean gcp2Separation(GraphData source, Set component, Set excluded, 
			Set cp, Set beta, VertexData center) {
		VertexData ld1 = null;
		for(Iterator i = component.iterator(); i.hasNext(); ) {
			VertexData u = (VertexData) i.next();
			if(excluded.contains(u))
				continue;
			if(ld1 == null || u.degree() < ld1.degree())
				ld1 = u;
		}
		
		cp.add(ld1);
		if(ld1.degree() == 1) {
			for(Iterator i = component.iterator(); i.hasNext(); ) {
				VertexData u = (VertexData) i.next();
				if(excluded.contains(u))
					continue;
				if(u.degree() == 1 && u.getID() != ld1.getID()) {
					cp.add(u);
					break;
				}
			}
		} else {
			VertexData ld2 = null;
			for(Iterator i = component.iterator(); i.hasNext(); ) {
				VertexData u = (VertexData) i.next();
				if(excluded.contains(u))
					continue;
				if(u.degree() == ld1.degree() && ld1.isAdjanced(u)) {
					cp.add(u);
					if(ld2 == null)
						ld2 = u;
				}
			}
			if(ld2 == null) {
				for(Iterator i = component.iterator(); i.hasNext(); ) {
					VertexData u = (VertexData) i.next();
					if(excluded.contains(u))
						continue;
					if(u.degree() == ld1.degree() + 2 && ld1.isAdjanced(u)) {
						ld2 = u;
						break;
					}
				}
				if(ld2 == null)
					return false;
			}
			for(Iterator i = component.iterator(); i.hasNext(); ) {
				VertexData u = (VertexData) i.next();
				if(excluded.contains(u))
					continue;
				if(u.degree() == ld1.degree() && ld2.isAdjanced(u))
					cp.add(u);
			}
			
		}
		if(!isCP(cp))
			return false;
		
		for(Iterator i = component.iterator(); i.hasNext(); ) {
			VertexData u = (VertexData) i.next();
			if(excluded.contains(u))
				continue;
			if(u.degree() > ld1.degree() && isAdjancedToAll(u, cp) && u.degree() < center.degree())
				beta.add(u);
		}
		
		return true;
	}
	
	private boolean isAdjanced(VertexData v, Collection c) {
		return adjanced(v, c) > 0;
	}
	
	private boolean isAdjancedToAll(VertexData v, Collection c) {
		return adjanced(v, c) == c.size();
	}
	
	private int adjanced(VertexData v, Collection c) {
		int a = 0;
		for(Iterator i = c.iterator(); i.hasNext(); ) {
			VertexData u = (VertexData) i.next();
			if(v.isAdjanced(u))
				a++;
		}
		return a;
	}
	
	private boolean isGCP(GraphData source, Set component) {
		int n = component.size();
		for(Iterator i = component.iterator(); i.hasNext(); ) {
			VertexData u = (VertexData) i.next();
			if(u.degree() != n - 1 && u.degree() != n - 2)
				return false;
		}
		return true;
	}
	
	private Collection buildComponents(GraphData graph) {
		Collection components = new Vector();
		Set marked = new HashSet(), toProcess = new HashSet(), q = new HashSet();
		
		for(Enumeration i = graph.vertices(); i.hasMoreElements(); ) {
			VertexData u = (VertexData) i.nextElement();
			toProcess.add(u);
		}
		
		while(!toProcess.isEmpty()) {
			Set component = new HashSet();
			VertexData u = (VertexData) toProcess.iterator().next();
			q.add(u);
			component.add(u);
			while(!q.isEmpty()) {
				u = (VertexData) q.iterator().next();
				q.remove(u);
				marked.add(u);
				toProcess.remove(u);
				for(Enumeration i = u.getNeighbors(); i.hasMoreElements(); ) {
					VertexData v = (VertexData) i.nextElement();
					if(!marked.contains(v)) {
						q.add(v);
						component.add(v);
					}
				}
			}
			components.add(component);
		}
		
		return components;
	}

	public String getName() {
		return "Generalized Line Graph Source";
	}

	public String getDescription() {
		return "Builds a source of a generalized line graph";
	}

	public String checkParameters() {
		return null;
	}
	
	private WeightVertexInt weight;

	public WeightVertexInt getWeight() {
		return weight;
	}
	
	public void setWeight(WeightVertexInt weight) {
		this.weight = weight;
	}
	
	private GraphData source;
	
	public GraphData getSource() {
		return source;
	}
	
	public void setSource(GraphData source) {
		this.source = source;
	}
	
	private class Point {
		public Point(double x, double y) {
			this.x = x;
			this.y = y;
		}
		public double x, y;
	}
	
}
